@@ -264,6 +264,8 @@ static int handle_packet(void)
 		} else if ( !f ) {
 			int i;
 			struct sigaction sa;
+			close(ap.pipefd);
+			close(ap.ioctlfd);
 			sa.sa_handler = SIG_DFL;
 			sigemptyset(&sa.sa_mask);
 			sa.sa_flags = 0;
