@@ -70,15 +70,15 @@ int close_parse(struct parse_mod *);
 
 /* mount module */
 
-#define AUTOFS_MOUNT_VERSION 2
+#define AUTOFS_MOUNT_VERSION 3
 
 #ifdef MODULE_MOUNT
 int mount_init(void **);
-int mount_mount(char *, char *, int, char *, char *, void *);
+int mount_mount(char *, char *, int, char *, char *, char *, void *);
 int mount_done(void *);
 #endif
 typedef (*mount_init_t)(void **);
-typedef (*mount_mount_t)(char *, char *, int, char *, char *, void *);
+typedef (*mount_mount_t)(char *, char *, int, char *, char *, char *, void *);
 typedef (*mount_done_t)(void *);
 struct mount_mod {
   mount_init_t  mount_init;
