@@ -47,7 +47,8 @@ int mount_init(void **context)
   return 0;
 }
 
-int mount_mount(char *root, char *name, int name_len, char *what, char *options, void *context)
+int mount_mount(char *root, char *name, int name_len, char *what,
+		char *fstype, char *options, void *context)
 {
   char *colon, **haddr, *fullpath;
   struct hostent *he;
@@ -128,8 +129,8 @@ int mount_mount(char *root, char *name, int name_len, char *what, char *options,
     sprintf(fullpath, "%s/%s", root, name);
     
     *colon = ':';
-    syslog(LOG_DEBUG, MODPREFIX "calling mkdir %s", name);
-    if ( mkdir(name, 0555) && errno != EEXIST ) {
+    syslog(LOG_DEBUG, MODPREFIX "calling mkdir %s", fullpath);
+    if ( mkdir(fullpath, 0555) && errno != EEXIST ) {
       syslog(LOG_NOTICE, MODPREFIX "mkdir %s failed: %m", name);
       free(fullpath);
       return 1;
