@@ -52,6 +52,25 @@ static struct substvar		/* Predefined variables: tail of link chain */
   sv_osrel  = { "OSREL",  un.release, &sv_osname },
   sv_osvers = { "OSVERS", un.version, &sv_osrel };
 
+/* Free all storage associated with this context */
+static void kill_context(struct parse_context *ctxt)
+{
+  struct substvar *sv, *nsv;
+
+  sv = ctxt->subst;
+  while ( sv != &sv_osvers ) {
+    nsv = sv->next;
+    free(sv);
+    sv = nsv;
+  }
+  
+  if ( ctxt->optstr )
+    free(ctxt->optstr);
+
+  free(ctxt);
+}
+
+/* Find the $-variable matching a certain string fragment */
 static struct substvar *findvar(struct substvar *sv, char *str, int len)
 {
   while ( sv ) {
@@ -62,9 +81,9 @@ static struct substvar *findvar(struct substvar *sv, char *str, int len)
   return NULL;
 }
 
-/* Parse a Sun-style map and return the length of the entry.
+/* $- and &-expand a Sun-style map entry and return the length of the entry.
    If "dst" is NULL, just count the length. */
-static int parsesunent(char *src, char *dst, char *key, struct substvar *svc)
+static int expandsunent(char *src, char *dst, char *key, struct substvar *svc)
 {
   struct substvar *sv;
   int len, l;
@@ -129,21 +148,52 @@ static int parsesunent(char *src, char *dst, char *key, struct substvar *svc)
   return len;
 }
 
-static void kill_context(struct parse_context *ctxt)
+/* Skip whitespace in a string; if we hit a #, consider the rest of the
+   entry a comment */
+char *skipspace(char *whence)
 {
-  struct substvar *sv, *nsv;
-
-  sv = ctxt->subst;
-  while ( sv != &sv_osvers ) {
-    nsv = sv->next;
-    free(sv);
-    sv = nsv;
+  while(1) {
+    switch(*whence) {
+    case ' ':
+    case '\b':
+    case '\t':
+    case '\n':
+    case '\v':
+    case '\f':
+    case '\r':
+      whence++;
+      break;
+    case '\0':
+    case '#':
+      return NULL;		/* End of string */
+    default:
+      return whence;
+    }
   }
-  
-  if ( ctxt->optstr )
-    free(ctxt->optstr);
+}
 
-  free(ctxt);
+/* Get the length of a chunk delimitered by whitespace */
+int chunklen(char *whence)
+{
+  int n = 0;
+  while(1) {
+    switch(*whence) {
+    case ' ':
+    case '\b':
+    case '\t':
+    case '\n':
+    case '\v':
+    case '\f':
+    case '\r':
+    case '#':
+    case '\0':
+      return n;
+    default:
+      break;
+    }
+    n++;
+    whence++;
+  }
 }
 
 int parse_init(int argc, char **argv, void **context)
@@ -237,6 +287,7 @@ int parse_init(int argc, char **argv, void **context)
 	return 1;
       }
       ctxt->optstr = noptstr;
+      syslog(LOG_DEBUG, MODPREFIX "init gathered options: %s", ctxt->optstr);
     }
   }
 
@@ -257,31 +308,80 @@ int parse_init(int argc, char **argv, void **context)
 int parse_mount(char *root, char *name, int name_len, char *mapent, void *context)
 {
   struct parse_context *ctxt = (struct parse_context *) context;
-  char *pmapent;
+  char *pmapent, *options, *noptions, *p;
   int mapent_len, rv;
+  int l, optlen;
 
-  mapent_len = parsesunent(mapent,NULL,name,ctxt->subst);
+  mapent_len = expandsunent(mapent,NULL,name,ctxt->subst);
   pmapent = malloc(mapent_len + 1);
   if (!pmapent) {
     syslog(LOG_ERR, MODPREFIX "malloc: %m");
     return 1;
   }
-  parsesunent(mapent,pmapent,name,ctxt->subst);
+  expandsunent(mapent,pmapent,name,ctxt->subst);
 
-  if ( pmapent[0] == '/' ) {
-    /* Uh-oh, a multientry; don't support now */
-    free(mapent);
-    syslog(LOG_NOTICE, "entry %s is a multipath entry", name);
-    return 1;
+  syslog(LOG_DEBUG, "expanded entry: %s", pmapent);
+
+  options = ctxt->optstr;	/* No options found */
+  optlen = options ? strlen(options) : 0;
+
+  p = pmapent;
+  while ( (p = skipspace(p)) ) {
+    if ( *p == '-' ) {
+      l = chunklen(++p);
+      if ( !l )
+	continue;
+      if ( options ) {
+	if ( options != ctxt->optstr ) {
+	  noptions = realloc(options, optlen+l+2);
+	  if ( !noptions ) {
+	    free(pmapent);
+	    free(options);
+	    syslog(LOG_ERR, MODPREFIX "realloc: %m");
+	    return 1;
+	  }
+	} else {
+	  noptions = malloc(optlen+l+2);
+	  if ( !noptions ) {
+	    free(pmapent);
+	    syslog(LOG_ERR, MODPREFIX "malloc: %m");
+	    return 1;
+	  }	  
+	  memcpy(noptions, ctxt->optstr, optlen+1);
+	}
+	noptions[optlen++] = ',';
+	memcpy(noptions+optlen, p, l);
+	noptions[optlen += l] = '\0';
+	options = noptions;
+      } else {
+	options = malloc(l+1);
+	if ( !options ) {
+	  free(pmapent);
+	  syslog(LOG_ERR, MODPREFIX "malloc: %m");
+	  return 1;
+	}
+	memcpy(options, p, l);
+	options[optlen = l] = '\0';
+      }
+      p += l;
+      syslog(LOG_DEBUG, MODPREFIX "gathered options: %s", options);
+    } else if ( *p == '/' ) {
+      /* Uh-oh, a multientry; don't support now, but ****FIXME**** */
+      free(pmapent);
+      if ( options != ctxt->optstr ) free(options);
+      syslog(LOG_NOTICE, MODPREFIX "entry %s is a multipath entry", name);
+      return 1;
+    } else {
+      p[chunklen(p)] = '\0';	/* We don't use anything after the entry */
+      syslog(LOG_DEBUG, MODPREFIX "core of entry: %s", p);
+      break;			/* We have the beef of the entry, run w/it */
+    }
   }
 
-  syslog(LOG_DEBUG, MODPREFIX "parsed %s -> %s", name, pmapent);
-
-  /*** FIXME: we still don't capture or even ignore in-map options ***/
-
-  rv = mount_nfs->mount_mount(root,name,name_len,pmapent,ctxt->optstr,
+  rv = mount_nfs->mount_mount(root, name, name_len, p, "nfs", options,
 			      mount_nfs->context);
   free(pmapent);
+  if ( options != ctxt->optstr ) free(options);
   return rv;
 }
 
